@@ -4,13 +4,13 @@
 # --------------------------------------------------------------------------------------------
 
 # pylint: disable=C0302
-
+from enum import Enum
 import re
 
-from enum import Enum
+# url parse package has different names in Python 2 and 3. 'six' package works cross-version.
+from six.moves.urllib.parse import (quote, urlparse)  # pylint: disable=import-error
 
 from azure.cli.core._profile import Profile
-from azure.cli.core.cloud import get_active_cloud
 from azure.cli.core.commands.client_factory import (
     get_mgmt_service_client,
     get_subscription_id)
@@ -32,9 +32,6 @@ from azure.mgmt.sql.models.sql_management_client_enums import (
 from azure.mgmt.resource import ResourceManagementClient
 from azure.mgmt.storage import StorageManagementClient
 
-# url parse package has different names in Python 2 and 3. 'six' package works cross-version.
-from six.moves.urllib.parse import (quote, urlparse)  # pylint: disable=import-error
-
 from ._util import (
     get_sql_servers_operations
 )
@@ -45,8 +42,8 @@ from ._util import (
 
 
 # Determines server location
-def get_server_location(server_name, resource_group_name):
-    server_client = get_sql_servers_operations(None)
+def _get_server_location(cli_ctx, server_name, resource_group_name):
+    server_client = get_sql_servers_operations(cli_ctx, None)
     # pylint: disable=no-member
     return server_client.get(
         server_name=server_name,
@@ -55,11 +52,11 @@ def get_server_location(server_name, resource_group_name):
 
 _DEFAULT_SERVER_VERSION = "12.0"
 
-
 ###############################################
 #                sql db                       #
 ###############################################
 
+
 # pylint: disable=too-few-public-methods
 class ClientType(Enum):
     ado_net = 'ado.net'
@@ -76,19 +73,20 @@ class ClientAuthenticationType(Enum):
     active_directory_integrated = 'ADIntegrated'
 
 
-def _get_server_dns_suffx():
+def _get_server_dns_suffx(cli_ctx):
     # Allow dns suffix to be overridden by environment variable for testing purposes
     from os import getenv
-    return getenv('_AZURE_CLI_SQL_DNS_SUFFIX', default=get_active_cloud().suffixes.sql_server_hostname)
+    return getenv('_AZURE_CLI_SQL_DNS_SUFFIX', default=cli_ctx.cloud.suffixes.sql_server_hostname)
 
 
 def db_show_conn_str(
+        cmd,
         client_provider,
         database_name='<databasename>',
         server_name='<servername>',
         auth_type=ClientAuthenticationType.sql_password.value):
 
-    server_suffix = _get_server_dns_suffx()
+    server_suffix = _get_server_dns_suffx(cmd.cli_ctx)
 
     conn_str_props = {
         'server': server_name,
@@ -180,14 +178,15 @@ def db_show_conn_str(
 
 # Helper class to bundle up database identity properties
 class DatabaseIdentity(object):  # pylint: disable=too-few-public-methods
-    def __init__(self, database_name, server_name, resource_group_name):
+    def __init__(self, cli_ctx, database_name, server_name, resource_group_name):
         self.database_name = database_name
         self.server_name = server_name
         self.resource_group_name = resource_group_name
+        self.cli_ctx = cli_ctx
 
     def id(self):
         return '/subscriptions/{}/resourceGroups/{}/providers/Microsoft.Sql/servers/{}/databases/{}'.format(
-            quote(get_subscription_id()),
+            quote(get_subscription_id(self.cli_ctx)),
             quote(self.resource_group_name),
             quote(self.server_name),
             quote(self.database_name))
@@ -196,13 +195,15 @@ class DatabaseIdentity(object):  # pylint: disable=too-few-public-methods
 # Creates a database or datawarehouse. Wrapper function which uses the server location so that
 # the user doesn't need to specify location.
 def _db_dw_create(
+        cli_ctx,
         client,
         db_id,
         raw,
         kwargs):
 
     # Determine server location
-    kwargs['location'] = get_server_location(
+    kwargs['location'] = _get_server_location(
+        cli_ctx,
         server_name=db_id.server_name,
         resource_group_name=db_id.resource_group_name)
 
@@ -218,6 +219,7 @@ def _db_dw_create(
 # Creates a database. Wrapper function which uses the server location so that the user doesn't
 # need to specify location.
 def db_create(
+        cmd,
         client,
         database_name,
         server_name,
@@ -232,14 +234,16 @@ def db_create(
                        ' `az sql dw create`.')
 
     return _db_dw_create(
+        cmd.cli_ctx,
         client,
-        DatabaseIdentity(database_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, server_name, resource_group_name),
         raw,
         kwargs)
 
 
 # Common code for special db create modes.
 def _db_create_special(
+        cli_ctx,
         client,
         source_db,
         dest_db,
@@ -247,7 +251,8 @@ def _db_create_special(
         kwargs):
 
     # Determine server location
-    kwargs['location'] = get_server_location(
+    kwargs['location'] = _get_server_location(
+        cli_ctx,
         server_name=dest_db.server_name,
         resource_group_name=dest_db.resource_group_name)
 
@@ -265,6 +270,7 @@ def _db_create_special(
 
 # Copies a database. Wrapper function to make create mode more convenient.
 def db_copy(
+        cmd,
         client,
         database_name,
         server_name,
@@ -283,15 +289,17 @@ def db_copy(
     kwargs['create_mode'] = 'Copy'
 
     return _db_create_special(
+        cmd.cli_ctx,
         client,
-        DatabaseIdentity(database_name, server_name, resource_group_name),
-        DatabaseIdentity(dest_name, dest_server_name, dest_resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, dest_name, dest_server_name, dest_resource_group_name),
         raw,
         kwargs)
 
 
 # Copies a replica. Wrapper function to make create mode more convenient.
 def db_create_replica(
+        cmd,
         client,
         database_name,
         server_name,
@@ -310,16 +318,44 @@ def db_create_replica(
 
     # Replica must have the same database name as the source db
     return _db_create_special(
+        cmd.cli_ctx,
         client,
-        DatabaseIdentity(database_name, server_name, resource_group_name),
-        DatabaseIdentity(database_name, partner_server_name, partner_resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, partner_server_name, partner_resource_group_name),
         raw,
         kwargs)
 
 
+# Renames a database.
+def db_rename(
+        cmd,
+        client,
+        database_name,
+        server_name,
+        resource_group_name,
+        new_name):
+
+    client.rename(
+        resource_group_name,
+        server_name,
+        database_name,
+        id=DatabaseIdentity(
+            cmd.cli_ctx,
+            new_name,
+            server_name,
+            resource_group_name
+        ).id())
+
+    return client.get(
+        resource_group_name,
+        server_name,
+        new_name)
+
+
 # Creates a database from a database point in time backup or deleted database backup.
 # Wrapper function to make create mode more convenient.
 def db_restore(
+        cmd,
         client,
         database_name,
         server_name,
@@ -341,16 +377,18 @@ def db_restore(
     kwargs['create_mode'] = 'Restore' if is_deleted else 'PointInTimeRestore'
 
     return _db_create_special(
+        cmd.cli_ctx,
         client,
-        DatabaseIdentity(database_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, server_name, resource_group_name),
         # Cross-server restore is not supported. So dest server/group must be the same as source.
-        DatabaseIdentity(dest_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, dest_name, server_name, resource_group_name),
         raw,
         kwargs)
 
 
 # Fails over a database. Wrapper function which uses the server location so that the user doesn't
 # need to specify replication link id.
+# pylint: disable=inconsistent-return-statements
 def db_failover(
         client,
         database_name,
@@ -433,6 +471,7 @@ def db_list_capabilities(
     return editions
 
 
+# pylint: disable=inconsistent-return-statements
 def db_delete_replica_link(
         client,
         database_name,
@@ -549,7 +588,8 @@ def db_update(
         instance,
         elastic_pool_name=None,
         max_size_bytes=None,
-        requested_service_objective_name=None):
+        requested_service_objective_name=None,
+        zone_redundant=None):
 
     # Verify edition
     if instance.edition.lower() == DatabaseEdition.data_warehouse.value.lower():
@@ -590,6 +630,8 @@ def db_update(
     # Set other properties
     instance.max_size_bytes = max_size_bytes or instance.max_size_bytes
 
+    instance.zone_redundant = zone_redundant
+
     return instance
 
 
@@ -604,14 +646,14 @@ def db_update(
 # because if it was a command line parameter then the customer would need to specify storage
 # resource group just to update some unrelated property, which is annoying and makes no sense to
 # the customer.
-def _find_storage_account_resource_group(name):
+def _find_storage_account_resource_group(cli_ctx, name):
     storage_type = 'Microsoft.Storage/storageAccounts'
     classic_storage_type = 'Microsoft.ClassicStorage/storageAccounts'
 
     query = "name eq '{}' and (resourceType eq '{}' or resourceType eq '{}')".format(
         name, storage_type, classic_storage_type)
 
-    client = get_mgmt_service_client(ResourceManagementClient)
+    client = get_mgmt_service_client(cli_ctx, ResourceManagementClient)
     resources = list(client.resources.list(filter=query))
 
     if not resources:
@@ -637,11 +679,12 @@ def _get_storage_account_name(storage_endpoint):
 
 # Gets storage account key by querying storage ARM API.
 def _get_storage_endpoint(
+        cli_ctx,
         storage_account,
         resource_group_name):
 
     # Get storage account
-    client = get_mgmt_service_client(StorageManagementClient)
+    client = get_mgmt_service_client(cli_ctx, StorageManagementClient)
     account = client.storage_accounts.get_properties(
         resource_group_name=resource_group_name,
         account_name=storage_account)
@@ -658,12 +701,13 @@ def _get_storage_endpoint(
 
 # Gets storage account key by querying storage ARM API.
 def _get_storage_key(
+        cli_ctx,
         storage_account,
         resource_group_name,
         use_secondary_key):
 
     # Get storage keys
-    client = get_mgmt_service_client(StorageManagementClient)
+    client = get_mgmt_service_client(cli_ctx, StorageManagementClient)
     keys = client.storage_accounts.list_keys(
         resource_group_name=resource_group_name,
         account_name=storage_account)
@@ -675,6 +719,7 @@ def _get_storage_key(
 
 # Common code for updating audit and threat detection policy
 def _db_security_policy_update(
+        cli_ctx,
         instance,
         enabled,
         storage_account,
@@ -690,8 +735,8 @@ def _db_security_policy_update(
     if storage_endpoint:
         instance.storage_endpoint = storage_endpoint
     if storage_account:
-        storage_resource_group = _find_storage_account_resource_group(storage_account)
-        instance.storage_endpoint = _get_storage_endpoint(storage_account, storage_resource_group)
+        storage_resource_group = _find_storage_account_resource_group(cli_ctx, storage_account)
+        instance.storage_endpoint = _get_storage_endpoint(cli_ctx, storage_account, storage_resource_group)
 
     # Set storage access key
     if storage_account_access_key:
@@ -708,9 +753,10 @@ def _db_security_policy_update(
         # function, but at least we tried.
         if not storage_account:
             storage_account = _get_storage_account_name(instance.storage_endpoint)
-            storage_resource_group = _find_storage_account_resource_group(storage_account)
+            storage_resource_group = _find_storage_account_resource_group(cli_ctx, storage_account)
 
         instance.storage_account_access_key = _get_storage_key(
+            cli_ctx,
             storage_account,
             storage_resource_group,
             use_secondary_key)
@@ -718,6 +764,7 @@ def _db_security_policy_update(
 
 # Update audit policy. Custom update function to apply parameters to instance.
 def db_audit_policy_update(
+        cmd,
         instance,
         state=None,
         storage_account=None,
@@ -733,6 +780,7 @@ def db_audit_policy_update(
 
     # Set storage-related properties
     _db_security_policy_update(
+        cmd.cli_ctx,
         instance,
         enabled,
         storage_account,
@@ -752,6 +800,7 @@ def db_audit_policy_update(
 
 # Update threat detection policy. Custom update function to apply parameters to instance.
 def db_threat_detection_policy_update(
+        cmd,
         instance,
         state=None,
         storage_account=None,
@@ -769,6 +818,7 @@ def db_threat_detection_policy_update(
 
     # Set storage-related properties
     _db_security_policy_update(
+        cmd.cli_ctx,
         instance,
         enabled,
         storage_account,
@@ -800,6 +850,7 @@ def db_threat_detection_policy_update(
 # Creates a datawarehouse. Wrapper function which uses the server location so that the user doesn't
 # need to specify location.
 def dw_create(
+        cmd,
         client,
         database_name,
         server_name,
@@ -812,8 +863,9 @@ def dw_create(
 
     # Create
     return _db_dw_create(
+        cmd.cli_ctx,
         client,
-        DatabaseIdentity(database_name, server_name, resource_group_name),
+        DatabaseIdentity(cmd.cli_ctx, database_name, server_name, resource_group_name),
         raw,
         kwargs)
 
@@ -857,6 +909,7 @@ def dw_update(
 # Creates an elastic pool. Wrapper function which uses the server location so that the user doesn't
 # need to specify location.
 def elastic_pool_create(
+        cmd,
         client,
         server_name,
         resource_group_name,
@@ -864,7 +917,8 @@ def elastic_pool_create(
         **kwargs):
 
     # Determine server location
-    kwargs['location'] = get_server_location(
+    kwargs['location'] = _get_server_location(
+        cmd.cli_ctx,
         server_name=server_name,
         resource_group_name=resource_group_name)
 
@@ -882,13 +936,15 @@ def elastic_pool_update(
         database_dtu_max=None,
         database_dtu_min=None,
         dtu=None,
-        storage_mb=None):
+        storage_mb=None,
+        zone_redundant=None):
 
     # Apply params to instance
     instance.database_dtu_max = database_dtu_max or instance.database_dtu_max
     instance.database_dtu_min = database_dtu_min or instance.database_dtu_min
     instance.dtu = dtu or instance.dtu
     instance.storage_mb = storage_mb or instance.storage_mb
+    instance.zone_redundant = zone_redundant
 
     return instance
 
@@ -1013,12 +1069,13 @@ def server_update(
 
 
 def server_ad_admin_set(
+        cmd,
         client,
         resource_group_name,
         server_name,
         **kwargs):
 
-    profile = Profile()
+    profile = Profile(cli_ctx=cmd.cli_ctx)
     sub = profile.get_subscription()
     kwargs['tenant_id'] = sub['tenantId']
 
@@ -1162,6 +1219,35 @@ def _get_server_key_name_from_uri(uri):
     return '{}_{}_{}'.format(vault, key, version)
 
 
+#####
+#           sql server dns-alias
+#####
+
+
+def server_dns_alias_set(
+        cmd,
+        client,
+        resource_group_name,
+        server_name,
+        dns_alias_name,
+        original_server_name,
+        original_subscription_id=None,
+        original_resource_group_name=None):
+
+    # Build the old alias id
+    old_alias_id = "/subscriptions/{}/resourceGroups/{}/providers/Microsoft.Sql/servers/{}/dnsAliases/{}".format(
+        quote(original_subscription_id or get_subscription_id(cmd.cli_ctx)),
+        quote(original_resource_group_name or resource_group_name),
+        quote(original_server_name),
+        quote(dns_alias_name))
+
+    return client.acquire(
+        resource_group_name=resource_group_name,
+        server_name=server_name,
+        dns_alias_name=dns_alias_name,
+        old_server_dns_alias_id=old_alias_id
+    )
+
 #####
 #           sql server encryption-protector
 #####
@@ -1190,33 +1276,3 @@ def encryption_protector_update(
             server_key_name=key_name
         )
     )
-
-
-#####
-#           sql server vnet-rule
-#####
-
-# Validates if a subnet id or name have been given by the user. If subnet id is given, vnet-name should not be provided.
-
-
-def validate_subnet(namespace):
-    from msrestazure.tools import resource_id, is_valid_resource_id
-
-    subnet = namespace.virtual_network_subnet_id
-    subnet_is_id = is_valid_resource_id(subnet)
-    vnet = namespace.vnet_name
-
-    if (subnet_is_id and not vnet) or (not subnet and not vnet):
-        pass
-    elif subnet and not subnet_is_id and vnet:
-        namespace.virtual_network_subnet_id = resource_id(
-            subscription=get_subscription_id(),
-            resource_group=namespace.resource_group_name,
-            namespace='Microsoft.Network',
-            type='virtualNetworks',
-            name=vnet,
-            child_type_1='subnets',
-            child_name_1=subnet)
-    else:
-        raise CLIError('incorrect usage: [--subnet ID | --subnet NAME --vnet-name NAME]')
-    delattr(namespace, 'vnet_name')
